[![Build Status][badge-travis-image]][badge-travis-url]

# Kong Response Transformer Advanced Plugin

Response Transformer Advanced.

This plugin is a proper superset of the Response Transformer plugin included
in Kong.

## Status

This plugin is **not** bundled with Kong Enterprise as of now.

## Configuration

### Parameters

Configuration parameters below are inherited from Kong Response Transformer:

| Form parameter | default | description
|----------------|---------|------------
| `name`         || The name of the plugin to configure; in this case, `response-transformer-advanced`
| `config.remove.headers` || List of header names. Unset the header(s) with the given name
| `config.remove.json` || List of property names. Remove the property from the JSON body if it is present
| `config.rename.headers` || List of header name `name1:name2` pairs. If a header with `name1` exists and `name2` is valid, rename header with `name2`.
| `config.replace.headers` || List of header `name:value` pairs. If and only if the header is already set, replace its old value with the new one. Ignored if the header is not already set
| `config.replace.json` || List of property `name:value` pairs. If and only if the parameter is already present, replace its old value with the new one. Ignored if the parameter is not already present
| `config.replace.json_types` || List of JSON type names. Specify the types of the JSON values returned when replacing JSON properties.
| `config.add.headers` || List of headername:value pairs. If and only if the header is not already set, set a new header with the given value. Ignored if the header is already set
| `config.add.json` || List of property:value pairs. If and only if the property is not present, add a new property with the given value to the JSON body. Ignored if the property is already present
| `config.add.json_types` || List of JSON type names. Specify the types of the JSON values returned when adding a new JSON property.
| `config.append.headers` || List of headername:value pairs. If the header is not set, set it with the given value. If it is already set, a new header with the same name and the new value will be set
| `config.append.json` || List of property:value pairs. If the property is not present in the JSON body, add it with the given value. If it is already present, the two values (old and new) will be aggregated in an array
| `config.replace.json_types` || List of JSON type names. Specify the types of the JSON values returned when appending JSON properties.
| `config.allow.json` || Set of parameter names. Only allows accepted parameters in the JSON response body.

New configuration directives:

| Form parameter | default | description
|----------------|---------|------------
| `config.remove.if_status` || List of response status codes or ranges to which the transformation will apply. Empty means all response codes
| `config.replace.if_status` || List of response status codes or ranges to which the transformation will apply. Empty means all response codes
| `config.replace.body` || String with which to replace the entire response body
| `config.add.if_status` || List of response status codes or ranges to which the transformation will apply. Empty means all response codes
| `config.append.if_status` || List of response status codes or ranges to which the transformation will apply. Empty means all response codes
| `config.allow.json` || Set of parameters name which should be allowed in response JSON body
| `config.transform.functions` || Set of lua functions to perform arbitrary transforms in a response JSON body
| `config.transform.if_status` || List of response status codes or ranges to which the arbitrary transformation will apply. Empty means all response codes

### Examples

- Add multiple headers by passing each header:value pair separately:

```
$ curl -X POST http://localhost:8001/routes/{route id}/plugins \
  --data "name=response-transformer-advanced" \
  --data "config.add.headers[1]=h1:v1" \
  --data "config.add.headers[2]=h2:v1"
```

- Add the headers, as above, but only if the response code is 200

```
$ curl -X POST http://localhost:8001/routes/{route id}/plugins \
  --data "name=response-transformer-advanced" \
  --data "config.add.headers[1]=h1:v1" \
  --data "config.add.headers[2]=h2:v1" \
  --data "config.add.if_status=200"
```

- Replace entire response body if response code is 500

```
$ curl -X POST http://localhost:8001/routes/{route id}/plugins \
  --data "name=response-transformer-advanced" \
  --data "config.replace.body='{\"error\": \"internal server error\"}'" \
  --data "config.replace.if_status=500"
```

- Explicitly specify the types of added JSON values if the response code is 500

```
$ curl -X POST http://localhost:8001/routes/{route id}/plugins \
  --data "name=response-transformer-advanced" \
  --data "config.add.json_types=number" \
  --data "config.replace.if_status=500"
```

**Note**: the plugin doesn't validate the value in `config.replace.body` against
the content type as defined in the `Content-Type` response header.

- Perform arbitrary transforms on a response body

Given some lua code in a file called wrangler.lua

```lua
--- wrangler.lua
return function (data)
  if type(data) ~= "table" then
    return data
  end

  -- remove foo key
  data["foo"] = nil

  -- add a new key
  data["foobar"] = "hello world"

  return data
end
```

And the plugin configured on a service

```
$ curl -X POST http://localhost:8001/services/{service}/plugins \
  --data "name=response-transformer-advanced" \
  --data "config.transform.functions=@wrangler.lua
```

Given this response

```json
{
  "foo": "bar",
  "something": "else"
}
```

The response will be transformed into

```json
{
  "something": "else",
  "foobar": "hello world"
}
```

[badge-travis-url]: https://travis-ci.com/Kong/kong-plugin-response-transformer-advanced/branches
[badge-travis-image]: https://travis-ci.com/Kong/kong-plugin-response-transformer-advanced.svg?token=BfzyBZDa3icGPsKGmBHb&branch=master
